/*
  Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
  License for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; see the file COPYING.LIB.  If not, write to the
  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  02110-1301, USA.
*/

#ifndef AKONADI_PUBLISHDIALOG_P_H
#define AKONADI_PUBLISHDIALOG_P_H

#include "publishdialog.h"
#include "ui_publishdialog_base.h"
#include <KContacts/Addressee>

namespace Akonadi {
class Q_DECL_HIDDEN PublishDialog::Private : public QObject
{
    Q_OBJECT
public:
    Private(PublishDialog *q);
    void insertAddresses(const KContacts::Addressee::List &list);

public Q_SLOTS:
    void addItem();
    void removeItem();
    void openAddressbook();
    void updateItem();
    void updateInput();

public:
    Ui::PublishDialog_base mUI;
    PublishDialog *q = nullptr;
};
}
#endif
